;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                               ;;
;;     DEVICE SETUP              ;;
;;                               ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   
   
use32
LLL equ (56+4)
BBB equ 25
                org     0x0
                db      'MENUET01'             ; 8 byte id
                dd      0x01                   ;
                dd      START                  ; program start
                dd      I_END                  ; program image end
                dd      0x80000                ; reguired amount of memory
                dd      0x7ffff                ; stack
                dd      I_PARAM,0              ; reserved
   
   
START:
 cmp [I_PARAM],'BOOT'
 jne no_param
_param:
    call _midibase
    call _cdbase
    call _hdbase
    call _sound_dma
    call _lba_read
    call _pci_acc
    call _f32p
    call _sb16
    call _wssp
    call _syslang
 mov eax,-1
 int 0x40
   
no_param:
    call  draw_window
   
still:
   
    mov  eax,10                 ; wait here for event
    int  0x40
   
    cmp  eax,1
    jz   red
    cmp  eax,2
    jz   key
    cmp  eax,3
    jz   button
   
    jmp  still
   
  red:
    jmp  START
   
  key:
    mov  eax,2
    int  0x40
    jmp  still
   
  button:
   
    mov  eax,17
    int  0x40
   
    cmp ah,100
    jne no_apply_all
    jmp _param
no_apply_all:
   
    cmp  ah,1                ; CLOSE APPLICATION
    jne  no_close
    mov  eax,-1
    int  0x40
  no_close:
   
    cmp  ah,11               ; SET MIDI BASE
    jnz  nosetbase1
    call _midibase
   nosetbase1:
    cmp  ah,12
    jnz  nomm
    mov  eax,[midibase]
    sub  eax,2
    mov  [midibase],eax
    call draw_infotext
  nomm:
    cmp  ah,13
    jnz  nomp
    mov  eax,[midibase]
    add  eax,2
    mov  [midibase],eax
    call draw_infotext
  nomp:
   
   
    cmp  ah,4                ; SET KEYBOARD
    jnz  nokm
    mov  eax,[keyboard]
    cmp  eax,0
    je   downuplbl
    dec  eax
    jmp  nodownup
   downuplbl:
    mov  eax,4
   nodownup:
    mov  [keyboard],eax
    call draw_infotext
  nokm:
    cmp  ah,5
    jnz  nokp
    mov  eax,[keyboard]
    cmp  eax,4
    je   updownlbl
    inc  eax
    jmp  noupdown
   updownlbl:
    mov  eax,0
   noupdown:
    mov  [keyboard],eax
    call draw_infotext
  nokp:
   
   
    cmp  ah,22                ; SET CD BASE
    jnz  nocm
    mov  eax,[cdbase]
    sub  eax,1
    dec  eax
    and  eax,3
    inc  eax
    mov  [cdbase],eax
    call draw_infotext
  nocm:
    cmp  ah,23
    jnz  nocp
    mov  eax,[cdbase]
    add  eax,1
    dec  eax
    and  eax,3
    inc  eax
    mov  [cdbase],eax
    call draw_infotext
  nocp:
    cmp  ah,21
    jnz  nocs
 call _cdbase
  nocs:
   
    cmp  ah,62              ; SET HD BASE
    jnz  hnocm
    mov  eax,[hdbase]
    sub  eax,1
    dec  eax
    and  eax,3
    inc  eax
    mov  [hdbase],eax
    call draw_infotext
  hnocm:
    cmp  ah,63
    jnz  hnocp
    mov  eax,[hdbase]
    add  eax,1
    dec  eax
    and  eax,3
    inc  eax
    mov  [hdbase],eax
    call draw_infotext
  hnocp:
    cmp  ah,61
    jnz  hnocs
    call _hdbase
  hnocs:
   
    cmp  ah,82              ; SET SOUND DMA
    jne  no_sdma_d
    mov  eax,[sound_dma]
    dec  eax
   sdmal:
    and  eax,3
    mov  [sound_dma],eax
    call draw_infotext
    jmp  still
  no_sdma_d:
    cmp  ah,83
    jne  no_sdma_i
    mov  eax,[sound_dma]
    inc  eax
    jmp  sdmal
  no_sdma_i:
    cmp  ah,81
    jne  no_set_sound_dma
    call _sound_dma
    jmp  still
  no_set_sound_dma:
   
    cmp  ah,92                   ; SET LBA READ
    jne  no_lba_d
    mov  eax,[lba_read]
    dec  eax
  slbal:
    and  eax,1
    mov  [lba_read],eax
    call draw_infotext
    jmp  still
   no_lba_d:
    cmp  ah,93
    jne  no_lba_i
    mov  eax,[lba_read]
    inc  eax
    jmp  slbal
  no_lba_i:
    cmp  ah,91
    jne  no_set_lba_read
    call _lba_read
    jmp  still
   no_set_lba_read:
   
   
    cmp  ah,102                   ; SET PCI ACCESS
     jne  no_pci_d
     mov  eax,[pci_acc]
    dec  eax
  pcip:
    and  eax,1
     mov  [pci_acc],eax
     call draw_infotext
    jmp  still
  no_pci_d:
    cmp  ah,103
    jne  no_pci_i
     mov  eax,[pci_acc]
     inc  eax
     jmp  pcip
   no_pci_i:
    cmp  ah,101
    jne  no_set_pci_acc
    call _pci_acc
    jmp  still
  no_set_pci_acc:
   
    cmp  ah,72                  ; SET FAT32 PARTITION
    jnz  fhnocm
    mov  eax,[f32p]
    sub  eax,1
    dec  eax
    and  eax,3
    inc  eax
    mov  [f32p],eax
    call draw_infotext
  fhnocm:
    cmp  ah,73
    jnz  fhnocp
    mov  eax,[f32p]
    add  eax,1
    dec  eax
    and  eax,3
    inc  eax
    mov  [f32p],eax
    call draw_infotext
  fhnocp:
    cmp  ah,71
    jnz  fhnocs
    call _f32p
  fhnocs:
   
    cmp  ah,32                  ; SET SOUND BLASTER 16 BASE
    jnz  nosbm
    mov  eax,[sb16]
    sub  eax,2
    mov  [sb16],eax
    call draw_infotext
  nosbm:
    cmp  ah,33
    jnz  nosbp
    mov  eax,[sb16]
    add  eax,2
    mov  [sb16],eax
    call draw_infotext
  nosbp:
    cmp  ah,31
    jnz  nosbs
    call _sb16
  nosbs:
   
    cmp  ah,52                  ; SET WINDOWS SOUND SYSTEM BASE
    jnz  nowssm
    mov  eax,[wss]
    sub  eax,1
    dec  eax
    and  eax,3
    inc  eax
    mov  [wss],eax
    call draw_infotext
  nowssm:
    cmp  ah,53
    jnz  nowssp
    mov  eax,[wss]
    add  eax,1
    dec  eax
    and  eax,3
    inc  eax
    mov  [wss],eax
    call draw_infotext
  nowssp:
    cmp  ah,51
    jnz  nowsss
    call _wssp
  nowsss:
   
    cmp  ah,42                ; SET SYSTEM LANGUAGE BASE
    jnz  nosysm
    mov  eax,[syslang]
    cmp  eax,1
    je   nosysm
    dec  eax
    mov  [syslang],eax
    call draw_infotext
  nosysm:
    cmp  ah,43
    jnz  nosysp
    mov  eax,[syslang]
    cmp  eax,4
    je   nosysp
    inc  eax
    mov  [syslang],eax
    call draw_infotext
  nosysp:
    cmp  ah,41
    jnz  nosyss
    call _syslang
  nosyss:
   
    cmp  ah,3                  ; SET KEYMAP
    jne  nosetkeyl
    mov  eax,[keyboard]
    cmp  eax,0
    jnz  nosetkeyle
    mov  eax,21       ; english
    mov  ebx,2
    mov  ecx,1
    mov  edx,en_keymap
    int  0x40
    mov  eax,21
    mov  ebx,2
    mov  ecx,2
    mov  edx,en_keymap_shift
    int  0x40
    mov  eax,21
    mov  ebx,2
    mov  ecx,9
    mov  edx,1
    int  0x40
    mov  eax,21
    mov  ebx,2
    mov  ecx,3
    mov  edx,alt_general
    int  0x40
  nosetkeyle:
    mov  eax,[keyboard]
    cmp  eax,1
    jnz  nosetkeylfi
    mov  eax,21       ; finnish
    mov  ebx,2
    mov  ecx,1
    mov  edx,fi_keymap
    int  0x40
    mov  eax,21
    mov  ebx,2
    mov  ecx,2
    mov  edx,fi_keymap_shift
    int  0x40
    mov  eax,21
    mov  ebx,2
    mov  ecx,9
    mov  edx,2
    int  0x40
    mov  eax,21
    mov  ebx,2
    mov  ecx,3
    mov  edx,alt_general
    int  0x40
  nosetkeylfi:
    mov  eax,[keyboard]
    cmp  eax,2
    jnz  nosetkeylge
    mov  eax,21       ; german
    mov  ebx,2
    mov  ecx,1
    mov  edx,ge_keymap
    int  0x40
    mov  eax,21
    mov  ebx,2
    mov  ecx,2
    mov  edx,ge_keymap_shift
    int  0x40
    mov  eax,21
    mov  ebx,2
    mov  ecx,9
    mov  edx,3
    int  0x40
    mov  eax,21
    mov  ebx,2
    mov  ecx,3
    mov  edx,alt_general
    int  0x40
  nosetkeylge:
    mov  eax,[keyboard]
    cmp  eax,3
    jnz  nosetkeylru
    mov  eax,21       ; russian
    mov  ebx,2
    mov  ecx,1
    mov  edx,ru_keymap
    int  0x40
    mov  eax,21
    mov  ebx,2
    mov  ecx,2
    mov  edx,ru_keymap_shift
    int  0x40
    mov  eax,21
    mov  ebx,2
    mov  ecx,3
    mov  edx,alt_general
    int  0x40
    mov  eax,21
    mov  ebx,2
    mov  ecx,9
    mov  edx,4
    int  0x40
  nosetkeylru:
    mov  eax,[keyboard]   ; french
    cmp  eax,4
    jnz  nosetkeylfr
    mov  eax,21
    mov  ebx,2
    mov  ecx,1
    mov  edx,fr_keymap
    int  0x40
    mov  eax,21
    mov  ebx,2
    mov  ecx,2
    mov  edx,fr_keymap_shift
    int  0x40
    mov  eax,21
    mov  ebx,2
    mov  ecx,3
    mov  edx,fr_keymap_alt_gr
    int  0x40
    mov  eax,21
    mov  ebx,2
    mov  ecx,9
    mov  edx,5
    int  0x40
  nosetkeylfr:
   
  nosetkeyl:
   
    jmp  still
   
   
   
draw_buttons:
   
    pusha
   
    shl  ecx,16
    add  ecx,12
    mov  ebx,(350-50)*65536+46+BBB
   
    mov  eax,8
    int  0x40
   
    mov  eax,8
    mov  ebx,(350-79)*65536+9
    inc  edx
    int  0x40
   
    mov  eax,8
    mov  ebx,(350-67)*65536+9
    inc  edx
    int  0x40
   
    popa
    ret
   
   
   
; ********************************************
; ******* WINDOW DEFINITIONS AND DRAW  *******
; ********************************************
   
   
draw_window:
   
    pusha
   
    mov  eax,12
    mov  ebx,1
    int  0x40
   
    mov  eax,0
    mov  ebx,40*65536+355+BBB
    mov  ecx,40*65536+256
    mov  edx,0x82111199
    mov  esi,0x805588dd
    mov  edi,0x005588dd
    int  0x40
   
    mov  eax,4
     mov  ebx,8*65536+7 ; 8
     mov  ecx,0x10ffffff
    mov  edx,labelt
    mov  esi,labellen-labelt
    int  0x40
   
    mov  eax,8                     ; CLOSE BUTTON
    mov  ebx,(355+BBB-19)*65536+12
    mov  ecx,5  *65536+12
    mov  edx,1
    mov  esi,0x005588dd
    int  0x40
   
    mov  eax,8                     ; APPLY ALL
    mov  ebx,(355-55)*65536+46+BBB
    mov  ecx,223*65536+24
    mov  edx,100
    mov  esi,0x005588dd
    int  0x40
   
    mov  esi,0x5580c0
   
    mov  edx,11
    mov  ecx,43
    call draw_buttons
   
    mov  edx,41
    mov  ecx,43+8*8
    call draw_buttons
   
    mov  edx,21
    mov  ecx,43+4*8
    call draw_buttons
   
    mov  edx,31
    mov  ecx,43+2*8
    call draw_buttons
   
    mov  edx,3
    mov  ecx,43+10*8
    call draw_buttons
   
    mov  edx,51
    mov  ecx,43+12*8
    call draw_buttons
   
    mov  edx,61
    mov  ecx,43+6*8
    call draw_buttons
   
    mov  edx,91
    mov  ecx,43+18*8
    call draw_buttons
   
    mov  edx,71
    mov  ecx,43+14*8
    call draw_buttons
   
    mov  edx,81
    mov  ecx,43+16*8
    call draw_buttons
   
    mov  edx,101
    mov  ecx,43+20*8
    call draw_buttons
   
    call draw_infotext
   
    mov  eax,12
    mov  ebx,2
    int  0x40
   
    popa
    ret
   
   
   
draw_infotext:
   
    pusha
   
    mov  eax,[keyboard]                       ; KEYBOARD
    cmp  eax,0
    jnz  noen
    mov  [text00+LLL*10+28],dword 'ENGL'
    mov  [text00+LLL*10+32],dword 'ISH '
  noen:
    cmp  eax,1
    jnz  nofi
    mov  [text00+LLL*10+28],dword 'FINN'
    mov  [text00+LLL*10+32],dword 'ISH '
  nofi:
    cmp  eax,2
    jnz  noge
    mov  [text00+LLL*10+28],dword 'GERM'
    mov  [text00+LLL*10+32],dword 'AN  '
  noge:
    cmp  eax,3
    jnz  nogr
    mov  [text00+LLL*10+28],dword 'RUSS'
    mov  [text00+LLL*10+32],dword 'IAN '
  nogr:
    cmp  eax,4
    jnz  nofr
    mov  [text00+LLL*10+28],dword 'FREN'
    mov  [text00+LLL*10+32],dword 'CH  '
  nofr:
   
   
    mov  eax,[syslang]                          ; SYSTEM LANGUAGE
    sub  eax,1
    cmp  eax,0
    jnz  noen5
    mov  [text00+LLL*8+28],dword 'ENGL'
    mov  [text00+LLL*8+32],dword 'ISH '
  noen5:
    cmp  eax,1
    jnz  nofi5
    mov  [text00+LLL*8+28],dword 'FINN'
    mov  [text00+LLL*8+32],dword 'ISH '
  nofi5:
    cmp  eax,2
    jnz  noge5
    mov  [text00+LLL*8+28],dword 'GERM'
    mov  [text00+LLL*8+32],dword 'AN  '
  noge5:
    cmp  eax,3
    jnz  nogr5
    mov  [text00+LLL*8+28],dword 'RUSS'
    mov  [text00+LLL*8+32],dword 'IAN '
  nogr5:
    cmp  eax,4
    jne  nofr5
    mov  [text00+LLL*8+28],dword 'FREN'
    mov  [text00+LLL*8+32],dword 'CH  '
  nofr5:
   
   
    mov  eax,[midibase]                          ; MIDI BASE
    xor  ebx,ebx
    mov  bl,al
    and  bl,15
    add  ebx,hex
    mov  cl,[ebx]
    mov  [text00+LLL*0+32],cl
    shr  eax,4
    xor  ebx,ebx
    mov  bl,al
    and  bl,15
    add  ebx,hex
    mov  cl,[ebx]
    mov  [text00+LLL*0+31],cl
    shr  eax,4
    xor  ebx,ebx
    mov  bl,al
    and  bl,15
    add  ebx,hex
    mov  cl,[ebx]
    mov  [text00+LLL*0+30],cl
   
   
    mov  eax,[sb16]                            ; SB16 BASE
    xor  ebx,ebx
    mov  bl,al
    and  bl,15
    add  ebx,hex
    mov  cl,[ebx]
    mov  [text00+LLL*2+32],cl
    shr  eax,4
    xor  ebx,ebx
    mov  bl,al
    and  bl,15
    add  ebx,hex
    mov  cl,[ebx]
    mov  [text00+LLL*2+31],cl
    shr  eax,4
    xor  ebx,ebx
    mov  bl,al
    and  bl,15
    add  ebx,hex
    mov  cl,[ebx]
    mov  [text00+LLL*2+30],cl
   
   
    mov  eax,[wss]                           ; WSS BASE
    cmp  eax,1
    jnz  nowss1
    mov  [wssp],dword 0x530
  nowss1:
    cmp  eax,2
    jnz  nowss2
    mov  [wssp],dword 0x608
  nowss2:
    cmp  eax,3
    jnz  nowss3
    mov  [wssp],dword 0xe80
  nowss3:
    cmp  eax,4
    jnz  nowss4
    mov  [wssp],dword 0xf40
  nowss4:
   
    mov  eax,[wssp]
    xor  ebx,ebx
    mov  bl,al
    and  bl,15
    add  ebx,hex
    mov  cl,[ebx]
    mov  [text00+LLL*12+32],cl
    shr  eax,4
    xor  ebx,ebx
    mov  bl,al
    and  bl,15
    add  ebx,hex
    mov  cl,[ebx]
    mov  [text00+LLL*12+31],cl
    shr  eax,4
    xor  ebx,ebx
    mov  bl,al
    and  bl,15
    add  ebx,hex
    mov  cl,[ebx]
    mov  [text00+LLL*12+30],cl
   
   
    mov  eax,[cdbase]                           ; CD BASE
    cmp  eax,1
    jnz  noe1
    mov  [text00+LLL*4+28],dword 'PRI.'
    mov  [text00+LLL*4+32],dword 'MAST'
    mov  [text00+LLL*4+36],dword 'ER  '
  noe1:
    cmp  eax,2
    jnz  nof1
    mov  [text00+LLL*4+28],dword 'PRI.'
    mov  [text00+LLL*4+32],dword 'SLAV'
    mov  [text00+LLL*4+36],dword 'E   '
  nof1:
    cmp  eax,3
    jnz  nog1
    mov  [text00+LLL*4+28],dword 'SEC.'
    mov  [text00+LLL*4+32],dword 'MAST'
    mov  [text00+LLL*4+36],dword 'ER  '
  nog1:
    cmp  eax,4
    jnz  nog2
    mov  [text00+LLL*4+28],dword 'SEC.'
    mov  [text00+LLL*4+32],dword 'SLAV'
    mov  [text00+LLL*4+36],dword 'E   '
  nog2:
   
   
    mov  eax,[hdbase]                         ; HD BASE
    cmp  eax,1
    jnz  hnoe1
    mov  [text00+LLL*6+28],dword 'PRI.'
    mov  [text00+LLL*6+32],dword 'MAST'
    mov  [text00+LLL*6+36],dword 'ER  '
  hnoe1:
    cmp  eax,2
    jnz  hnof1
    mov  [text00+LLL*6+28],dword 'PRI.'
    mov  [text00+LLL*6+32],dword 'SLAV'
    mov  [text00+LLL*6+36],dword 'E   '
  hnof1:
    cmp  eax,3
    jnz  hnog1
    mov  [text00+LLL*6+28],dword 'SEC.'
    mov  [text00+LLL*6+32],dword 'MAST'
    mov  [text00+LLL*6+36],dword 'ER  '
  hnog1:
    cmp  eax,4
    jnz  hnog2
    mov  [text00+LLL*6+28],dword 'SEC.'
    mov  [text00+LLL*6+32],dword 'SLAV'
    mov  [text00+LLL*6+36],dword 'E   '
  hnog2:
   
   
    mov  eax,[f32p]                       ; FAT32 PARTITION
    add  al,48
    mov  [text00+LLL*14+28],al
   
    mov  eax,[sound_dma]                  ; SOUND DMA
    add  eax,48
    mov  [text00+LLL*16+28],al
   
    mov  eax,[lba_read]                   ; LBA READ
    mov  ebx,'  '
    cmp  eax,1
    je   lbar
    mov  ebx,' '
  lbar:
    mov  [text00+LLL*18+28],ebx
   
    mov  eax,[pci_acc]                   ; PCI ACCESS
    mov  ebx,'  '
    cmp  eax,1
    je   lbar2
    mov  ebx,' '
  lbar2:
    mov  [text00+LLL*20+28],ebx
   
    mov  eax,13
    mov  ebx,175*65536+85
    mov  ecx,40*65536+182
    mov  edx,0x80111199-19
    int  0x40
   
    mov  edx,text00
    xor  edi,edi
    mov  ebx,10*65536+44 ; 45
  newline:
    mov  eax,4
    mov  ecx,0xffffff
    mov  esi,LLL
    int  0x40
    add  ebx,8
    add  edx,LLL
    inc  edi
    cmp  [edx],byte 'x'
    jnz  newline
   
    popa
    ret
   
_midibase:
    mov  eax,21
    mov  ebx,1
    mov  ecx,[midibase]
    int  0x40
 ret
   
_cdbase:
    mov  eax,21
    mov  ebx,3
    mov  ecx,[cdbase]
    int  0x40
 ret
   
_hdbase:
    mov  eax,21
    mov  ebx,7
    mov  ecx,[hdbase]
    int  0x40
    ret
   
_sound_dma:
    mov  eax,21
    mov  ebx,10
    mov  ecx,[sound_dma]
    int  0x40
    ret
   
_lba_read:
    mov  eax,21
    mov  ebx,11
    mov  ecx,[lba_read]
    int  0x40
    ret
   
_pci_acc:
    mov  eax,21
    mov  ebx,12
    mov  ecx,[pci_acc]
    int  0x40
 ret
   
_f32p:
    mov  eax,21
    mov  ebx,8
    mov  ecx,[f32p]
    int  0x40
 ret
   
_sb16:
    mov  eax,21
    mov  ebx,4
    mov  ecx,[sb16]
    int  0x40
    ret
   
_wssp:
    mov  eax,21
    mov  ebx,6
    mov  ecx,[wssp]
    int  0x40
 ret
   
_syslang:
    mov  eax,21
    mov  ebx,5
    mov  ecx,[syslang]
    int  0x40
 ret
   
; DATA AREA
   
   
text00:
   
    db ' MIDI  ROLAND MPU-401 : 0x320           - +   ਬ '
    db '                                                            '
    db ' SoundBlaster 16      : 0x240           - +   ਬ '
    db '                                                            '
    db ' CD-ROM              : PRI.SLAVE       - +   ਬ '
    db '                                                            '
    db ' -1                 : PRI.MASTER      - +   ਬ '
    db '                                                            '
    db ' ⥬              : ENGLISH         - +   ਬ '
    db '                                                            '
    db '᪫       : ENGLISH         - +   ਬ '
    db '                                                            '
    db ' WSS                  : 0x200           - +   ਬ '
    db '                                                            '
    db ' FAT32  -1      : 1               - +   ਬ '
    db '                                                            '
    db '㪮  DMA        : 1               - +   ਬ '
    db '                                                            '
    db '⥭ LBA                : OFF             - +   ਬ '
    db '                                                            '
    db '  設 PCI         : OFF             - +   ਬ '
    db '                                                            '
    db '                                                            '
    db ':    FAT-32              '
    db '           !                               '
    db 'x                                                           '
   
   
keyboard  dd 0x0
midibase  dd 0x320
cdbase    dd 0x2
sb16      dd 0x220
syslang   dd 0x1
wss       dd 0x1
wssp      dd 0x0
hdbase    dd 0x1
f32p      dd 0x1
sound_dma dd 0x1
lba_read  dd 0x0
pci_acc   dd 0x0
   
   
labelt:
    db   '  MENUET'
labellen:
   
   
hex db   '0123456789ABCDEF'
   
alt_general:
   
     db   ' ',27
     db   ' @ $  {[]}\ ',8,9
     db   '            ',13
     db   '             ',0,'           ',0,'4',0,' '
     db   '             ',180,178,184,'6',176,'7'
     db   179,'8',181,177,183,185,182
     db   'ABCD',255,'FGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
   
   
en_keymap:
   
     db   '6',27
     db   '1234567890-=',8,9
     db   'qwertyuiop[]',13
     db   '~asdfghjkl;',39,96,0,'\zxcvbnm,./',0,'45 '
     db   '@234567890123',180,178,184,'6',176,'7'
     db   179,'8',181,177,183,185,182
     db   'AB<D',255,'FGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
   
   
en_keymap_shift:
   
     db   '6',27
     db   '!@#$%^&*()_+',8,9
     db   'QWERTYUIOP{}',13
     db   '~ASDFGHJKL:"~',0,'|ZXCVBNM<>?',0,'45 '
     db   '@234567890123',180,178,184,'6',176,'7'
     db   179,'8',181,177,183,185,182
     db   'AB>D',255,'FGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
   
   
fr_keymap:
   
     db   '6',27
     db   '&"',39,'(-_)=',8,9
     db   'azertyuiop^$',13
     db   '~qsdfghjklm',0,0,'*wxcvbn,;:!',0,'45 '
     db   '@234567890123',180,178,184,'6',176,'7'
     db   179,'8',181,177,183,185,182
     db   'AB<D',255,'FGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
   
   
   
fr_keymap_shift:
   
   
     db   '6',27
     db   '1234567890+',8,9
     db   'AZERTYUIOP',13
     db   '~QSDFGHJKLM%',0,'WXCVBN?./',0,'45 '
     db   '@234567890123',180,178,184,'6',176,'7'
     db   179,'8',181,177,183,185,182
     db   'AB>D',255,'FGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
   
   
fr_keymap_alt_gr:
   
   
     db   '6',27
     db   28,'~#{[|\^@]}',8,9
     db   'azertyuiop^$',13
     db   '~qsdfghjklm',0,0,'*wxcvbn,;:!',0,'45 '
     db   '@234567890123',180,178,184,'6',176,'7'
     db   179,'8',181,177,183,185,182
     db   'AB<D',255,'FGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
   
   
   
   
fi_keymap:
   
     db   '6',27
     db   '1234567890+[',8,9
     db   'qwertyuiop',192,'~',13
     db   '~asdfghjkl',194,193,'1',0,39,'zxcvbnm,.-',0,'45 '
     db   '@234567890123',180,178,184,'6',176,'7'
     db   179,'8',181,177,183,185,182
     db   'AB<D',255,'FGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
   
   
fi_keymap_shift:
   
     db   '6',27
     db   '!"#%&/()=?]',8,9
     db   'QWERTYUIOP',200,'~',13
     db   '~ASDFGHJKL',202,201,'1',0,'*ZXCVBNM;:_',0,'45 '
     db   '@234567890123',180,178,184,'6',176,'7'
     db   179,'8',181,177,183,185,182
     db   'AB>D',255,'FGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
   
   
   
ge_keymap:
   
     db   '6',27
     db   '1234567890?[',8,9
     db   'qwertzuiop',203,'~',13
     db   '~asdfghjkl',194,193,'1',0,39,'yxcvbnm,.-',0,'45 '
     db   '@234567890123',180,178,184,'6',176,'7'
     db   179,'8',181,177,183,185,182
     db   'AB<D',255,'FGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
   
   
ge_keymap_shift:
   
     db   '6',27
     db   '!"#$%&/()=',197,']',8,9
     db   'QWERTZUIOP',195,'~',13
     db   '~ASDFGHJKL',202,201,'1',0,'*YXCVBNM;:_',0,'45 '
     db   '@234567890123',180,178,184,'6',176,'7'
     db   179,'8',181,177,183,185,182
     db   'AB>D',255,'FGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
   
ru_keymap:
   
     db   '6',27
     db   '1234567890-=',8,9
     db   '㪥',13
     db   0,"뢠஫"
     db   0xf1, '-/'
     db   "ᬨ",'.-','45 '
     db   '@234567890123',180,178,184,'6',176,'7'
     db   179,'8',181,177,183,185,182
     db   'AB<D',255,'FGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
   
   
   
ru_keymap_shift:
   
     db   '6',27
     db   '!"N;%:?*()_+',8,0
     db   "",13
     db   0,""
     db   0xf0, '-\'
     db   "",',-','45 '
     db   '@234567890123',180,178,184,'6',176,'7'
     db   179,'8',181,177,183,185,182
     db   'AB>D',255,'FGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
     db   'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
   
I_PARAM dd 0
   
I_END:
   
   
   
   
   
   
   
   
   